/*
Testing defines to be shared between various tests.

Copyright 2009 Canonical Ltd.

Authors:
    Ted Gould <ted@canonical.com>

This program is free software: you can redistribute it and/or modify it 
under the terms of the GNU General Public License version 3, as published 
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranties of 
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#define  TEST_ID                   "my-id"
#define  TEST_ICON_NAME            "my-icon-name"
#define  TEST_ATTENTION_ICON_NAME  "my-attention-icon-name"
#define  TEST_STATE                APP_INDICATOR_STATUS_ACTIVE
#define  TEST_STATE_S              "Active"
#define  TEST_CATEGORY             APP_INDICATOR_CATEGORY_APPLICATION_STATUS
#define  TEST_CATEGORY_S           "ApplicationStatus"
#define  TEST_OBJECT               "/an/object/path/to/use"

